import logging
import os
import typing as t
from pathlib import Path

import pandas as pd

import joblib
from sklearn.model_selection import train_test_split
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import LabelEncoder

from ows_language_model import model as m
from ows_language_model.config import config

_logger = logging.getLogger(__name__)



def save_pipeline(model) -> None:
    """Persist keras model to disk."""

    joblib.dump(model.named_steps['dataset'], config.PIPELINE_PATH)
    joblib.dump(model.named_steps['cnn_model'].classes_, config.CLASSES_PATH)
    model.named_steps['cnn_model'].model.save(str(config.MODEL_PATH))

    remove_old_pipelines(
        files_to_keep=[config.MODEL_FILE_NAME, config.ENCODER_FILE_NAME,
                       config.PIPELINE_FILE_NAME, config.CLASSES_FILE_NAME])


def load_pipeline() -> Pipeline:
    """Load a Pytorch Pipeline from disk."""

    return


def load_encoder() -> LabelEncoder:
    encoder = joblib.load(config.ENCODER_PATH)

    return encoder


def remove_old_pipelines(*, files_to_keep: t.List[str]) -> None:
    """
    Remove old model pipelines, models, encoders and classes.

    This is to ensure there is a simple one-to-one
    mapping between the package version and the model
    version to be imported and used by other applications.
    """
    do_not_delete = files_to_keep + ['__init__.py']
    for model_file in Path(config.TRAINED_MODEL_DIR).iterdir():
        if model_file.name not in do_not_delete:
            model_file.unlink()



from sklearn.pipeline import Pipeline

from ows_language_model.config import config
from ows_language_model.processing import preprocessors as pp
from ows_language_model import model


pipe = Pipeline([
                ('dataset', pp.CreateDataset()),
                ('language_model', model)])



import logging

import pandas as pd

from ows_language_model import __version__ as _version
from ows_language_model.processing import data_management as dm

_logger = logging.getLogger(__name__)
PIPELINE = dm.load_pipeline()


def make_single_prediction(*, input_text):
    """Make a single prediction using the saved model pipeline.

        Args:
            input_text: variable length sequence of text

        Returns
            Dictionary with both raw next-word predictions and readable values.
        """
    
    predictions = input_text #PIPELINE.predict(input_text)
    
    _logger.info(f'Made prediction: {predictions}'
                 f' with model version: {_version}')

    return dict(predictions=predictions,
                version=_version)


def make_bulk_prediction(*, text_df: pd.Series) -> dict:
    """Make multiple predictions using the saved model pipeline.

    Currently, this function is primarily for testing purposes,
    allowing us to pass in a series of text examples

    Args:
        images_df: Pandas series of images

    Returns
        Dictionary with both raw predictions and their classifications.
    """

    _logger.info(f'received input df: {text_df}')

    predictions = PIPELINE.predict(text_df)

    _logger.info(f'Made predictions: {predictions}'
                 f' with model version: {_version}')

    return dict(predictions=predictions,
                version=_version)



from ows_language_model.config import config

# TODO: define model here

# defines all callbacks, scheduling and builds model here

model = None

if __name__ == '__main__':
    # could print summary
    pass
